/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.activity.manager;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.control.activity.task.IllegalTaskException;
import org.objectweb.dream.control.activity.task.Task;
import org.objectweb.dream.control.activity.task.TaskStoppedListener;
import org.objectweb.dream.control.content.BasicContentController;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Basic task manager implementation
 */
public class TaskManagerImpl extends AbstractComponent
    implements
      TaskManager,
      EndOfTask
{

  Map<Task, TaskState>   registeredTask                       = new HashMap<Task, TaskState>();

  // ---------------------------------------------------------------------------
  // client interfaces
  // ---------------------------------------------------------------------------

  SchedulerManager       schedulerManagerItf;

  /** Name of the content controller client interface of this component. */
  static final String    ACTIVITY_CONTENT_CONTROLLER_ITF_NAME = "activities-content";
  BasicContentController contentControllerItf;

  // ---------------------------------------------------------------------------
  // Implementation of the TaskManager interface
  // ---------------------------------------------------------------------------

  /**
   * @see TaskManager#registerTask(Task, Map)
   */
  public synchronized void registerTask(Task task, Map hints)
      throws IllegalTaskException
  {
    if (registeredTask.containsKey(task))
    {
      throw new IllegalTaskException(task, "Task already registered");
    }
    // add task as sub component
    try
    {
      contentControllerItf.addFcSubComponent(task);
    }
    catch (Exception e)
    {
      Error.bug(logger, e);
    }

    TaskState state = new TaskState(task, hints);
    registeredTask.put(task, state);

    // schedule task.
    Object o = schedulerManagerItf.scheduleTask(task, hints);
    task.registered(o);
  }

  /**
   * @see TaskManager#unregisterTask(Task)
   */
  public synchronized void unregisterTask(Task task)
      throws IllegalTaskException, InterruptedException
  {
    TaskState state = registeredTask.get(task);
    if (state == null)
    {
      throw new IllegalTaskException(task, "Unknown Task");
    }
    if (state.state == TaskState.RUNNING)
    {
      synchronized (state)
      {
        interruptTask(task, state);
        while (state.state != TaskState.STOPPED)
        {
          state.wait();
        }
      }
    }
    else
    {
      doUnregister(task);
    }
  }

  /**
   * @see TaskManager#interruptTask(Task, TaskStoppedListener)
   */
  public synchronized void interruptTask(Task task, TaskStoppedListener listener)
      throws IllegalTaskException
  {
    TaskState state = registeredTask.get(task);
    if (state == null)
    {
      throw new IllegalTaskException(task, "Unknown Task");
    }
    if (state.state == TaskState.INTERRUPTED)
    {
      throw new IllegalTaskException(task, "task already interrupted");
    }
    state.state = TaskState.INTERRUPTED;
    state.listener = listener;
    schedulerManagerItf.interruptTask(task);
  }

  /**
   * @see TaskManager#getTasks()
   */
  public Task[] getTasks()
  {
    Task[] tasks = new Task[registeredTask.size()];
    registeredTask.keySet().toArray(tasks);
    return tasks;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the EndOfTask interface
  // ---------------------------------------------------------------------------

  /**
   * @see EndOfTask#taskEnded(Task)
   */
  public void taskEnded(Task taskItf)
  {
    TaskState state = registeredTask.get(taskItf);
    if (state == null)
    {
      Error.bug(logger);
    }
    doUnregister(taskItf);
    if (state.listener != null)
    {
      state.listener.taskStopped(taskItf);
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{SchedulerManager.ITF_NAME,
        ACTIVITY_CONTENT_CONTROLLER_ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(SchedulerManager.ITF_NAME))
    {
      schedulerManagerItf = (SchedulerManager) serverItf;
    }
    else if (clientItfName.equals(ACTIVITY_CONTENT_CONTROLLER_ITF_NAME))
    {
      contentControllerItf = (BasicContentController) serverItf;
    }
  }

  // ---------------------------------------------------------------------------
  // Utility method
  // ---------------------------------------------------------------------------

  private void doUnregister(Task task)
  {
    try
    {
      schedulerManagerItf.unscheduleTask(task);
      contentControllerItf.removeFcSubComponent(task);
    }
    catch (Exception e)
    {
      Error.bug(logger, e);
    }
    registeredTask.remove(task);
    task.unregistered();
  }

  // ---------------------------------------------------------------------------
  // Inner class
  // ---------------------------------------------------------------------------

  class TaskState implements TaskStoppedListener
  {
    static final int    RUNNING     = 0;
    static final int    INTERRUPTED = 1;
    static final int    STOPPED     = 2;

    Task                task;
    Map                 hints;
    int                 state;
    TaskStoppedListener listener;

    TaskState(Task task, Map hints)
    {
      this.task = task;
      this.hints = hints;
      state = RUNNING;
    }

    /**
     * @see TaskStoppedListener#taskStopped(Task)
     */
    public synchronized void taskStopped(Task task)
    {
      state = STOPPED;
      notifyAll();
    }
  }
}
