/**
 * DREAM
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Matthieu Leclercq
 */

package org.objectweb.dream.control.content;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.julia.ChainedNoSuchInterfaceException;
import org.objectweb.fractal.julia.Controller;
import org.objectweb.fractal.julia.InitializationContext;

/**
 * Provides a basic implementation of the {@link BasicContentController}
 * interface.
 */
public abstract class BasicContentControllerMixin
    implements
      Controller,
      BasicContentController
{
  // -------------------------------------------------------------------------
  // Private constructor
  // -------------------------------------------------------------------------

  private BasicContentControllerMixin()
  {
  }

  // -------------------------------------------------------------------------
  // Fields and methods added and overridden by the mixin class
  // -------------------------------------------------------------------------

  /**
   * The internal interfaces of the component to which this controller object
   * belongs.
   */

  public Map<String, Object> fcInternalInterfaces;

  /**
   * The sub components of the component to which this controller object
   * belongs.
   */

  public List<Object>        fcSubComponents;

  /**
   * Initializes the fields of this mixin and then calls the overridden method.
   * 
   * @param ic information about the component to which this controller object
   *          belongs.
   * @throws InstantiationException if the initialization fails.
   */
  @SuppressWarnings("unchecked")
  public void initFcController(final InitializationContext ic)
      throws InstantiationException
  {
    fcInternalInterfaces = ic.internalInterfaces;
    _super_initFcController(ic);
  }

  /**
   * @see BasicContentController#getFcInternalInterfaces()
   */
  public Object[] getFcInternalInterfaces()
  {
    if (fcInternalInterfaces == null)
    {
      return new Object[0];
    }
    // returns the names of all the public interfaces in fcInternalInterfaces
    // interfaces whose name begins with '/' are private
    int size = 0;
    String[] names = new String[fcInternalInterfaces.size()];
    names = fcInternalInterfaces.keySet().toArray(names);
    for (int i = 0; i < names.length; ++i)
    {
      if (names[i].charAt(0) != '/')
      {
        ++size;
      }
    }
    int index = 0;
    Object[] itfs = new Object[size];
    for (int i = 0; i < names.length; ++i)
    {
      if (names[i].charAt(0) != '/')
      {
        itfs[index++] = fcInternalInterfaces.get(names[i]);
      }
    }
    return itfs;
  }

  /**
   * @see BasicContentController#getFcInternalInterface(String)
   */
  public Object getFcInternalInterface(final String interfaceName)
      throws NoSuchInterfaceException
  {
    Object itf;
    if (fcInternalInterfaces == null)
    {
      itf = null;
    }
    else
    {
      itf = fcInternalInterfaces.get(interfaceName);
    }
    if (itf == null)
    {
      throw new ChainedNoSuchInterfaceException(null, _this_weaveableOptC,
          interfaceName);
    }
    return itf;
  }

  /**
   * @see BasicContentController#getFcSubComponents()
   */
  public Object[] getFcSubComponents()
  {
    if (fcSubComponents == null)
    {
      return new Object[0];
    }
    return fcSubComponents.toArray(new Object[fcSubComponents.size()]);
  }

  /**
   * @see BasicContentController#addFcSubComponent(Object)
   */
  public void addFcSubComponent(Object subComponent)
      throws IllegalContentException, IllegalLifeCycleException
  {
    if (fcSubComponents == null)
    {
      fcSubComponents = new ArrayList<Object>();
    }
    fcSubComponents.add(subComponent);
  }

  /**
   * @see BasicContentController#removeFcSubComponent(Object)
   */
  public void removeFcSubComponent(Object subComponent)
      throws IllegalContentException, IllegalLifeCycleException
  {
    if (fcSubComponents != null)
    {
      fcSubComponents.remove(subComponent);
    }
  }

  // -------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // -------------------------------------------------------------------------

  /**
   * The <tt>weaveableOptC</tt> field required by this mixin. This field is
   * supposed to reference the {@link Component}interface of the component to
   * which this controller object belongs.
   */

  public Component _this_weaveableOptC;

  /**
   * The {@link Controller#initFcController initFcController}method overridden
   * by this mixin.
   * 
   * @param ic information about the component to which this controller object
   *          belongs.
   * @throws InstantiationException if the initialization fails.
   */

  public abstract void _super_initFcController(InitializationContext ic)
      throws InstantiationException;
}
