/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message;

import java.lang.ref.Reference;
import java.util.Iterator;

import org.objectweb.dream.pool.Recyclable;

/**
 * Reference to a message created and held by a {@link MessageManager}.
 */
public abstract class Message implements Recyclable, ReferenceCounter
{
  private final MessageManager             owner;
  private int                              msgReferenceCounter = 0;
  private Reference< ? extends Recyclable> recyclableReference;

  protected Message(final MessageManager owner)
  {
    this.owner = owner;
  }

  final MessageManager getOwner()
  {
    return owner;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Recyclable interface
  // ---------------------------------------------------------------------------

  /**
   * Used only by message manager.
   * 
   * @see Recyclable#setReference(Reference)
   */
  public void setReference(final Reference< ? extends Recyclable> reference)
  {
    this.recyclableReference = reference;
  }

  /**
   * Used only by message manager.
   * 
   * @see Recyclable#getReference()
   */
  public Reference< ? extends Recyclable> getReference()
  {
    return recyclableReference;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the ReferenceCounter interface
  // ---------------------------------------------------------------------------

  /**
   * Used only by message manager.
   * 
   * @see ReferenceCounter#decrementReferenceCounter()
   */
  public boolean decrementReferenceCounter()
  {
    return (--msgReferenceCounter) == 0;
  }

  /**
   * Used only by message manager.
   * 
   * @see ReferenceCounter#incrementReferenceCounter()
   */
  public void incrementReferenceCounter()
  {
    msgReferenceCounter++;
  }

  // ---------------------------------------------------------------------------
  // Abstract methods
  // ---------------------------------------------------------------------------

  protected abstract boolean isEmpty();
  
  protected abstract AbstractChunk getChunk(String name);

  protected abstract void addChunk(String name, AbstractChunk chunk);

  protected abstract void removeChunk(String name);

  protected abstract Iterator<String> getNameIterator();

  protected abstract void addSubMessage(Message subMessage);

  protected abstract boolean isSubMessage(Message subMessage);

  protected abstract boolean removeSubMessage(Message subMessage);

  protected abstract Iterator<Message> getSubMessageIterator();
}
