/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.pool;

import java.util.HashSet;
import java.util.Set;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * ObjectPool that checks that objects are recycled only once. This component
 * can be used for debug purpose.
 * 
 * @see org.objectweb.dream.pool.ObjectPoolImpl
 */
public class ObjectPoolDebugImpl extends AbstractComponent
    implements
      ObjectPool,
      ObjectPoolAttributeController
{
  private int               newInstancesCount = 0;
  /** The pool of objects. */
  private Recyclable        pool[];

  /** The number of elements stored in the pool. */
  private int               elementCount      = 0;

  /** The capacity of the pool. */
  private int               capacity;

  private final Object      lock              = new Object();

  Set<Recyclable>           recycledObjects   = new HashSet<Recyclable>();
  Set<Recyclable>           createdObjects    = new HashSet<Recyclable>();

  // ---------------------------------------------------------------------------
  // Client interface
  // ---------------------------------------------------------------------------

  private RecyclableFactory recyclableFactoryItf;

  // ---------------------------------------------------------------------------
  // Implementation of the ObjectPool interface
  // ---------------------------------------------------------------------------

  /**
   * @see ObjectPool#newInstance()
   */
  public Recyclable newInstance()
  {
    Recyclable obj = null;
    synchronized (lock)
    {
      if (elementCount == 0)
      {
        if (logger.isLoggable(BasicLevel.INFO))
        {
          newInstancesCount++;
          logger.log(BasicLevel.INFO,
              "Creates new instance. created instances = " + newInstancesCount);
        }
        obj = recyclableFactoryItf.newInstance();
      }
      else
      {
        elementCount -= 1;
        obj = pool[elementCount];
        pool[elementCount] = null;
        if (!recycledObjects.remove(obj))
        {
          Error.bug(logger);
        }
      }
      if (!createdObjects.add(obj))
      {
        Error.bug(logger);
      }
    }
    return obj;
  }

  /**
   * @see ObjectPool#recycleInstance(Recyclable)
   */
  public void recycleInstance(Recyclable recyclable)
  {
    if (recyclable != null)
    {
      recyclable.recycle();
      synchronized (lock)
      {
        if (!createdObjects.contains(recyclable))
        {
          logger.log(BasicLevel.ERROR,
              "Recycling an object that has not been created by this pool",
              new Exception());
          Error.bug(logger);
        }

        if (!createdObjects.remove(recyclable))
        {
          Error.bug(logger);
        }

        if (recycledObjects.contains(recyclable))
        {
          logger.log(BasicLevel.ERROR, "Recycling an already recycled object",
              new Exception());
          Error.bug(logger);
        }

        if (!recycledObjects.add(recyclable))
        {
          Error.bug(logger);
        }

        if (elementCount == pool.length)
        {
          if (logger.isLoggable(BasicLevel.INFO))
          {
            logger.log(BasicLevel.INFO, "Pool Full created instances = "
                + newInstancesCount);
          }
          return;
        }
        pool[elementCount] = recyclable;
        elementCount += 1;
      }
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the PoolAttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see ObjectPoolAttributeController#getCapacity()
   */
  public int getCapacity()
  {
    return capacity;
  }

  /**
   * @see ObjectPoolAttributeController#setCapacity(int)
   */
  public void setCapacity(int capacity)
  {
    // TODO try to re-use previous instance
    synchronized (lock)
    {
      this.capacity = capacity;
      pool = new Recyclable[capacity];
      elementCount = 0;
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{RecyclableFactory.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  @Override
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(RecyclableFactory.ITF_NAME))
    {
      recyclableFactoryItf = (RecyclableFactory) serverItf;
    }
  }

}
