/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.pool;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This class provides a basic implementation of {@link ObjectPool} interface.
 * Recycled instances are pooled in a fixed size array. Instances are created
 * using a {@link RecyclableFactory} client interface.
 */
public class ObjectPoolImpl extends AbstractComponent
    implements
      ObjectPool,
      ObjectPoolAttributeController
{
  /** The pool of objects. */
  private Recyclable        pool[];

  /** The number of elements stored in the pool. */
  private int               elementCount = 0;

  /** The capacity of the pool. */
  private int               capacity;

  private final Object      lock         = new Object();

  // ---------------------------------------------------------------------------
  // Client interface
  // ---------------------------------------------------------------------------

  private RecyclableFactory recyclableFactoryItf;

  // ---------------------------------------------------------------------------
  // Implementation of the ObjectPool interface
  // ---------------------------------------------------------------------------

  /**
   * @see ObjectPool#newInstance()
   */
  public Recyclable newInstance()
  {
    Recyclable obj = null;
    synchronized (lock)
    {
      if (elementCount != 0)
      {
        elementCount -= 1;
        obj = pool[elementCount];
        pool[elementCount] = null;
        return obj;
      }
    }
    if (logger.isLoggable(BasicLevel.INFO))
    {
      logger.log(BasicLevel.INFO, "Creates new instance");
    }
    obj = recyclableFactoryItf.newInstance();
    return obj;
  }

  /**
   * @see ObjectPool#recycleInstance(Recyclable)
   */
  public void recycleInstance(final Recyclable recyclable)
  {
    if (recyclable != null /* && objectClass.isInstance(recyclable) */)
    {
      recyclable.recycle();
      synchronized (lock)
      {
        if (elementCount == pool.length)
        {
          if (logger.isLoggable(BasicLevel.INFO))
          {
            logger.log(BasicLevel.INFO, "Pool Full");
          }
          return;
        }
        pool[elementCount] = recyclable;
        elementCount += 1;
      }
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the PoolAttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see ObjectPoolAttributeController#getCapacity()
   */
  public int getCapacity()
  {
    return capacity;
  }

  /**
   * @see ObjectPoolAttributeController#setCapacity(int)
   */
  public void setCapacity(int capacity)
  {
    // TODO try to re-use previous instance
    synchronized (lock)
    {
      this.capacity = capacity;
      pool = new Recyclable[capacity];
      elementCount = 0;
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{RecyclableFactory.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  @Override
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(RecyclableFactory.ITF_NAME))
    {
      recyclableFactoryItf = (RecyclableFactory) serverItf;
    }
  }

}
